@@ -58,6 +58,9 @@
 
 #define MAX_INCLUDE_DEPTH 10
 
+/* The largest possible "name+'='+value" property pair length on Windows. */
+#define MAX_ENV_PAIR_LEN 32767
+
 EnvSrc *baseEnvSrc = NULL;
 
 /** Stores the time that the property file began to be loaded. */
@@ -495,10 +498,18 @@ static int loadPropertiesCallback(void *callbackParam, const TCHAR *fileName, in
  * @param properties Properties structure to load into.
  * @param filename File to load the properties from.
  * @param preload TRUE if this is a preload call that should have supressed error output.
+ * @param argCommand Argument passed to the binary.
+ * @param originalWorkingDir Working directory of the binary at the moment it was launched.
+ * @param isDebugging Flag that controls whether or not debug output will be logged.
  *
  * @return TRUE if there were any problems, FALSE if successful.
  */
-int loadProperties(Properties *properties, const TCHAR* filename, int preload) {
+int loadProperties(Properties *properties,
+                   const TCHAR* filename,
+                   int preload,
+                   const TCHAR *argCommand,
+                   const TCHAR *originalWorkingDir,
+                   int isDebugging) {
     /* Store the time that the property file began to be loaded. */
     #ifdef WIN32
     struct _timeb timebNow;
@@ -519,7 +530,7 @@ int loadProperties(Properties *properties, const TCHAR* filename, int preload) {
     nowTM = localtime(&now);
     memcpy(&loadPropertiesTM, nowTM, sizeof(struct tm));
 
-    loadResult = configFileReader(filename, FALSE, loadPropertiesCallback, properties, TRUE, preload);
+    loadResult = configFileReader(filename, FALSE, loadPropertiesCallback, properties, TRUE, preload, argCommand, originalWorkingDir, properties->warnedVarMap, properties->logWarnings, properties->logWarningLogLevel, isDebugging);
 
     /* Any failure is a failure in the root. */
     switch (loadResult) {
@@ -547,6 +558,7 @@ Properties* createProperties() {
     properties->last = NULL;
     properties->warnedVarMap = newHashMap(8);
     if (!properties->warnedVarMap) {
+        disposeProperties(properties);
         return NULL;
     }
     return properties;
@@ -674,7 +686,7 @@ int setEnvInner(const TCHAR *name, const TCHAR *value) {
 #ifdef WIN32
  #if defined(WRAPPER_USE_PUTENV_S)
             if (_tputenv_s(name, TEXT("")) == EINVAL) {
-                _tprintf(TEXT("Unable to clear environment variable: %s\n"), name);
+                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL, TEXT("Unable to clear the '%s' environment variable."), name);
                 result = TRUE;
             }
  #else
@@ -687,7 +699,7 @@ int setEnvInner(const TCHAR *name, const TCHAR *value) {
                 _sntprintf(envBuf, len, TEXT("%s="), name);
                 /* The memory pointed to by envBuf should only be freed if this is UNICODE. */
                 if (_tputenv(envBuf)) { 	 
-                    _tprintf(TEXT("Unable to clear environment variable: %s\n"), name); 	 
+                    log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL, TEXT("Unable to clear the '%s' environment variable."), name);
                     result = TRUE; 	 
                 }
             }
@@ -703,8 +715,8 @@ int setEnvInner(const TCHAR *name, const TCHAR *value) {
                 _sntprintf(envBuf, len, TEXT("%s="), name);
                 /* The memory pointed to by envBuf should only be freed if this is UNICODE. */
                 if (_tputenv(envBuf)) { 	 
-                    _tprintf(TEXT("Unable to clear environment variable: %s\n"), name); 	 
-                    result = TRUE; 	 
+                    log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL, TEXT("Unable to clear the '%s' environment variable."), name);
+                    result = TRUE;
                 }
   #ifdef UNICODE
                 free(envBuf);
@@ -721,21 +733,26 @@ int setEnvInner(const TCHAR *name, const TCHAR *value) {
 #ifdef WIN32
  #if defined(WRAPPER_USE_PUTENV_S)
             if (_tputenv_s(name, value) == EINVAL) {
-                _tprintf(TEXT("Unable to set environment variable: %s=%s\n"), name, value);
+                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL, TEXT("Unable to set '%s% environment variable to: %s"), name, value);
                 result = TRUE;
             }
  #else
             len = _tcslen(name) + 1 + _tcslen(value) + 1;
-            envBuf = malloc(sizeof(TCHAR) * len);
-            if (!envBuf) {
-                outOfMemory(TEXT("SEI"), 2); 	 
+            if (len > MAX_ENV_PAIR_LEN) {
+                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL, TEXT("Unable to set the '%s' environment variable because total pair length of %d is longer than maximum for the OS of %d."), name, len,  MAX_ENV_PAIR_LEN);
                 result = TRUE;
             } else {
-                _sntprintf(envBuf, len, TEXT("%s=%s"), name, value);
-                /* The memory pointed to by envBuf should only be freed if this is UNICODE. */
-                if (_tputenv(envBuf)) { 	 
-                    _tprintf(TEXT("Unable to set environment variable: %s=%s\n"), name, value); 	 
-                    result = TRUE; 	 
+                envBuf = malloc(sizeof(TCHAR) * len);
+                if (!envBuf) {
+                    outOfMemory(TEXT("SEI"), 2); 	 
+                    result = TRUE;
+                } else {
+                    _sntprintf(envBuf, len, TEXT("%s=%s"), name, value);
+                    /* The memory pointed to by envBuf should only be freed if this is UNICODE. */
+                    if (_tputenv(envBuf)) {
+                        log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL, TEXT("Unable to set environment variable: %s=%s"), name, value);
+                        result = TRUE; 	 
+                    }
                 }
             }
  #endif
@@ -750,8 +767,8 @@ int setEnvInner(const TCHAR *name, const TCHAR *value) {
                 _sntprintf(envBuf, len, TEXT("%s=%s"), name, value);
                 /* The memory pointed to by envBuf should only be freed if this is UNICODE. */
                 if (_tputenv(envBuf)) { 	 
-                    _tprintf(TEXT("Unable to set environment variable: %s=%s\n"), name, value); 	 
-                    result = TRUE; 	 
+                    log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL, TEXT("Unable to set the '%s' environment variable to: %s"), name, value);
+                    result = TRUE;
                 }
   #ifdef UNICODE
                 free(envBuf);
@@ -759,7 +776,7 @@ int setEnvInner(const TCHAR *name, const TCHAR *value) {
             }
  #else
             if (_tsetenv(name, value, TRUE)) {
-                _tprintf(TEXT("Unable to set environment variable: %s=%s\n"), name, value);
+                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL, TEXT("Unable to set the '%s' environment variable to: %s"), name, value);
                 result = TRUE;
             }
  #endif
@@ -784,8 +801,11 @@ int setEnvInner(const TCHAR *name, const TCHAR *value) {
  *
  * @param name Name of the variable being set.
  * @param value Value to be set, NULL to clear it.
- * @param source Where the variable came from.  If value is WRAPPER_ENV_SOURCE_PARENT
- *               then the value may be NULL and will never be set to the environment.
+ * @param source Where the variable came from.
+ *               Must be one of ENV_SOURCE_PARENT, ENV_SOURCE_APPLICATION, ENV_SOURCE_CONFIG,
+ *                 or ENV_SOURCE_REG_SYSTEM or ENV_SOURCE_REG_ACCOUNT on Windows.
+ *               If value is ENV_SOURCE_PARENT then the value may be NULL and will never be
+ *                 set to the environment.
  *
  * Return TRUE if there were any problems, FALSE otherwise.
  */
@@ -1210,6 +1230,7 @@ Property* addProperty(Properties *properties, const TCHAR* filename, int lineNum
         }
 
         if (property->value == NULL) {
+            free(propertyValueTrim);
             return NULL;
         }
         /* Store the final flag */
@@ -1236,7 +1257,7 @@ Property* addProperty(Properties *properties, const TCHAR* filename, int lineNum
                 log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_STATUS, TEXT("set default env('%s', '%s')"),
                     property->name + 12, property->value);
 #endif
-                setEnv(property->name + 12, property->value, (internal ? ENV_SOURCE_WRAPPER : ENV_SOURCE_CONFIG));
+                setEnv(property->name + 12, property->value, (internal ? ENV_SOURCE_APPLICATION : ENV_SOURCE_CONFIG));
             } else {
 #ifdef _DEBUG
                 log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_STATUS, TEXT(
@@ -1255,7 +1276,7 @@ Property* addProperty(Properties *properties, const TCHAR* filename, int lineNum
             log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_STATUS, TEXT("set env('%s', '%s')"),
                 property->name + 4, property->value);
 #endif
-            setEnv(property->name + 4, property->value, (internal ? ENV_SOURCE_WRAPPER : ENV_SOURCE_CONFIG));
+            setEnv(property->name + 4, property->value, (internal ? ENV_SOURCE_APPLICATION : ENV_SOURCE_CONFIG));
         }
     }
     free(propertyValueTrim);
