@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2013 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -56,6 +56,18 @@
 
 #include "property.h"
 
+#ifndef WIN32
+ /*
+  * Mac OSX 10.5 does not define the environ variable.  This is work around for that.
+  */
+ #ifdef MACOSX
+  #include <crt_externs.h>
+  #define environ *_NSGetEnviron();
+ #endif
+
+extern char** environ;
+#endif
+
 /* The following define will enable debug output of the code to parse the JVM output. */
 /*#define DEBUG_CHILD_OUTPUT*/
 
@@ -78,9 +90,13 @@
 #define WRAPPER_TIMER_FAST_THRESHOLD (2 * 24 * 3600 * 1000 / WRAPPER_TICK_MS) /* Default to 2 days. */
 #define WRAPPER_TIMER_SLOW_THRESHOLD (2 * 24 * 3600 * 1000 / WRAPPER_TICK_MS) /* Default to 2 days. */
 
-#define WRAPPER_BACKEND_TYPE_UNKNOWN 0 /* Unknown type. */
-#define WRAPPER_BACKEND_TYPE_SOCKET  1 /* Use a loopback socket to communicate. */
-#define WRAPPER_BACKEND_TYPE_PIPE    2 /* Use a pair of pipes to communicate. */
+#define WRAPPER_BACKEND_ERROR_NEXT     99 /* value returned when there is an error trying to connect to a socket and we should try the next way to connect */
+#define WRAPPER_BACKEND_TYPE_UNKNOWN   0 /* Unknown type. */
+#define WRAPPER_BACKEND_TYPE_SOCKET_V4 0x01
+#define WRAPPER_BACKEND_TYPE_SOCKET_V6 0x02
+#define WRAPPER_BACKEND_TYPE_SOCKET    (WRAPPER_BACKEND_TYPE_SOCKET_V4 | WRAPPER_BACKEND_TYPE_SOCKET_V6)
+#define WRAPPER_BACKEND_TYPE_PIPE      0x04 /* Use a pair of pipes to communicate. */
+#define WRAPPER_BACKEND_TYPE_AUTO      (WRAPPER_BACKEND_TYPE_SOCKET | WRAPPER_BACKEND_TYPE_PIPE)
 
 #define WRAPPER_WSTATE_STARTING  51 /* Wrapper is starting.  Remains in this state
                                      *  until the JVM enters the STARTED state or
@@ -185,13 +201,6 @@ typedef unsigned int TICKS;
 typedef unsigned long TICKS;
 #endif
 
-#define WRAPPER_ENV_SOURCE_PARENT      1
-#define WRAPPER_ENV_SOURCE_WRAPPER     2
-#ifdef WIN32
-#define WRAPPER_ENV_SOURCE_REG_SYSTEM  4
-#define WRAPPER_ENV_SOURCE_REG_ACCOUNT 8
-#endif
-
 #ifdef WIN32
 /* Defines the maximum number of service manager control events that can be queued in a single loop. */
 #define CTRL_CODE_QUEUE_SIZE 26 /* Can enqueue one less than this count at any time. */
@@ -254,8 +263,10 @@ struct WrapperConfig {
     int     commandLogLevel;        /* The log level to use when logging the java command. */
     int     printJVMVersion;        /* tells the Wrapper to create a temp JVM to query the version, before starting the java application */
 #ifdef WIN32
+    TCHAR   *jvmVersionCommand;     /* Command used to launch the JVM and request its version */
     TCHAR   *jvmCommand;            /* Command used to launch the JVM */
 #else /* UNIX */
+    TCHAR   **jvmVersionCommand;    /* Command used to launch the JVM and request its version */
     TCHAR   **jvmCommand;           /* Command used to launch the JVM */
 #endif
     int     detachStarted;          /* TRUE if the JVM process should be detached once it has started. */
@@ -277,7 +288,7 @@ struct WrapperConfig {
     int     shutdownTimeout;        /* Number of seconds the wrapper will wait for a JVM to shutdown */
     int     jvmExitTimeout;         /* Number of seconds the wrapper will wait for a JVM to process to terminate */
     int     jvmCleanupTimeout;      /* Number of seconds the wrapper will allow for its post JVM shudown cleanup. */
-    int     jvmTerminateTimeout;      /* Number of seconds the wrapper will allow for the JVM to respond to TerminateProcess request. */
+    int     jvmTerminateTimeout;    /* Number of seconds the wrapper will allow for the JVM to respond to TerminateProcess request. */
 #ifdef WIN32
     int     javaIOBufferSize;       /* Size of the pipe buffer to use for java I/O. */
 #endif
@@ -531,6 +542,24 @@ extern int wrapperProtocolRead();
 /******************************************************************************
  * Utility Functions
  *****************************************************************************/
+
+/**
+ * Does any necessary post processing on the command string.
+ *  This function assumes that command has been malloced.  It will either return
+ *  the string as is, or return a modified string.  When a modified string is
+ *  returned the orignal command buffer will always be freed.
+ *
+ * 1) Replace the first instance of the %WRAPPER_COMMAND_FILLER_N% environment
+ *    variable so that the total command length will be equal to or greater than
+ *    the length specified by N.  The padding will be the length plus a series of
+ *    Xs terminated by a single Y.  This is mainly for testing.
+ *
+ * @param command The original command.
+ *
+ * @return The modifed command.
+ */
+extern TCHAR *wrapperPostProcessCommandElement(TCHAR *command);
+
 /**
  * Test function to pause the current thread for the specified amount of time.
  *  This is used to test how the rest of the Wrapper behaves when a particular
@@ -753,11 +782,6 @@ extern void wrapperMaintainSignals();
 extern TCHAR* findPathOf(const TCHAR *exe, const TCHAR* name);
 #endif
 
-/**
- * Gets the error code for the last operation that failed.
- */
-extern int wrapperGetLastError();
-
 /**
  * Execute initialization code to get the wrapper set up.
  */
@@ -808,8 +832,10 @@ extern void wrapperPauseBeforeExecute();
 
 /**
  * Launches a JVM process and store it internally
+ *
+ * @return TRUE if there were any problems.  When this happens the Wrapper will not try to restart.
  */
-extern void wrapperExecute();
+extern int wrapperExecute();
 
 /**
  * Returns a tick count that can be used in combination with the
@@ -897,6 +923,13 @@ extern void wrapperDetachJava();
  */
 extern void wrapperStopProcess(int exitCode, int force);
 
+/**
+ * Depending on the current state, we want to change the exact message displayed when restarting the JVM.
+ *
+ * The logic here needs to match that in wrapperRestartProcess.
+ */
+extern const TCHAR *wrapperGetRestartProcessMessage();
+
 /**
  * Used to ask the state engine to shut down the JVM.
  */
@@ -1058,6 +1091,6 @@ struct LoadParameterFileCallbackParam {
 };
 
 #ifdef CUNIT
-extern void testJavaAdditionalParamSuite(void);
+extern void tsJAP_testJavaAdditionalParamSuite(void);
 #endif /* CUNIT */
 #endif
