@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2013 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -11,34 +11,32 @@
 
 /**
  * Author:
- *   Leif Mortenson <leif@tanukisoftware.com>
+ *   Tanuki Software Development Team <support@tanukisoftware.com>
  */
 
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #ifdef WIN32
-#include <errno.h>
-#include <tchar.h>
-#include <io.h>
+ #include <errno.h>
+ #include <tchar.h>
+ #include <io.h>
+ #include <winsock.h>
 #else
-#include <stdlib.h>
-#include <string.h>
-#include <glob.h>
-#include <unistd.h>
-#include <limits.h>
-#include <langinfo.h>
-#if defined(IRIX)
-#define PATH_MAX FILENAME_MAX
-#endif
+ #include <stdlib.h>
+ #include <string.h>
+ #include <unistd.h>
+ #include <limits.h>
+ #include <langinfo.h>
+ #if defined(IRIX)
+  #define PATH_MAX FILENAME_MAX
+ #endif
 #endif
 
 #include "wrapper_file.h"
 #include "logger.h"
 #include "wrapper_i18n.h"
-#include "wrapper.h"
-
-#define FILES_CHUNK 5
+#include "property.h"
 
 #ifndef TRUE
 #define TRUE -1
@@ -50,551 +48,18 @@
 
 #define MAX_INCLUDE_DEPTH 10
 
-/**
- * Returns a valid sort mode given a name: "TIMES", "NAMES_ASC", "NAMES_DEC".
- *  In the event of an invalid value, TIMES will be returned.
- */
-int wrapperFileGetSortMode(const TCHAR *modeName) {
-    if (strcmpIgnoreCase(modeName, TEXT("NAMES_ASC")) == 0) {
-        return WRAPPER_FILE_SORT_MODE_NAMES_ASC;
-    } else if (strcmpIgnoreCase(modeName, TEXT("NAMES_DEC")) == 0) {
-        return WRAPPER_FILE_SORT_MODE_NAMES_DEC;
-    } else {
-        return WRAPPER_FILE_SORT_MODE_TIMES;
-    }
-}
-
-
-#ifdef WIN32
-int sortFilesTimes(TCHAR **files, __time64_t *fileTimes, int cnt) {
-#else
-int sortFilesTimes(TCHAR **files, time_t *fileTimes, int cnt) {
-#endif
-    int i, j;
-    TCHAR *temp;
-#ifdef WIN32
-    __time64_t tempTime;
-#else
-    time_t tempTime;
-#endif
-
-    for (i = 0; i < cnt; i++) {
-        for (j = 0; j < cnt - 1; j++) {
-            if (fileTimes[j] < fileTimes[j + 1]) {
-                temp = files[j + 1];
-                tempTime = fileTimes[j + 1];
-
-                files[j + 1] = files[j];
-                fileTimes[j + 1] = fileTimes[j];
-
-                files[j] = temp;
-                fileTimes[j] = tempTime;
-            }
-        }
-    }
-
-    return TRUE;
-}
-
-/**
- * Compares two strings.  Returns 0 if they are equal, -1 if file1 is bigger, 1 if file2 is bigger.
- */
-int compareFileNames(const TCHAR *file1, const TCHAR *file2) {
-    int pos1, pos2;
-    TCHAR c1, c2;
-    int numeric1, numeric2;
-    long int num1, num2;
-    int afterNumber = FALSE;
-
-    pos1 = 0;
-    pos2 = 0;
-
-    while (TRUE) {
-        c1 = file1[pos1];
-        c2 = file2[pos2];
-        /*printf("     file1[%d]=%d, file2[%d]=%d\n", pos1, c1, pos2, c2);*/
-
-        /* Did we find the null. */
-        if (c1 == 0) {
-            if (c2 == 0) {
-                return 0;
-            } else {
-                return 1;
-            }
-        } else {
-            if (c2 == 0) {
-                return -1;
-            } else {
-                /* Continue. */
-            }
-        }
-
-        /* We have two characters. */
-        numeric1 = (c1 >= TEXT('0') && c1 <= TEXT('9'));
-        numeric2 = (c2 >= TEXT('0') && c2 <= TEXT('9'));
-
-        /* See if one or both of the strings is numeric. */
-        if (numeric1) {
-            if (numeric2) {
-                /* Both are numeric, we need to start comparing the two file names as integer values. */
-                num1 = c1 - TEXT('0');
-                c1 = file1[pos1 + 1];
-                while (c1 >= TEXT('0') && c1 <= TEXT('9')) {
-                    num1 = num1 * 10 + (c1 - TEXT('0'));
-                    pos1++;
-                    c1 = file1[pos1 + 1];
-                }
-
-                num2 = c2 - TEXT('0');
-                c2 = file2[pos2 + 1];
-                while (c2 >= TEXT('0') && c2 <= TEXT('9')) {
-                    num2 = num2 * 10 + (c2 - TEXT('0'));
-                    pos2++;
-                    c2 = file2[pos2 + 1];
-                }
-
-                /*printf("     num1=%ld, num2=%ld\n", num1, num2);*/
-                if (num1 > num2) {
-                    return -1;
-                } else if (num2 > num1) {
-                    return 1;
-                } else {
-                    /* Equal, continue. */
-                }
-                afterNumber = TRUE;
-            } else {
-                /* 1 is numeric, 2 is not. */
-                if (afterNumber) {
-                    return -1;
-                } else {
-                    return 1;
-                }
-            }
-        } else {
-            if (numeric2) {
-                /* 1 is not, 2 is numeric. */
-                if (afterNumber) {
-                    return 1;
-                } else {
-                    return -1;
-                }
-            } else {
-                /* Neither is numeric. */
-            }
-        }
-
-        /* Compare the characters as is. */
-        if (c1 > c2) {
-            return -1;
-        } else if (c2 > c1) {
-            return 1;
-        } else {
-            /* Equal, continue. */
-            if (c1 == TEXT('.') || c1 == TEXT('-') || c1 == TEXT('_')) {
-            } else {
-                afterNumber = FALSE;
-            }
-        }
-
-        pos1++;
-        pos2++;
-    }
-}
-
-int sortFilesNamesAsc(TCHAR **files, int cnt) {
-    int i, j;
-    TCHAR *temp;
-    int cmp;
-
-    for (i = 0; i < cnt; i++) {
-        for (j = 0; j < cnt - 1; j++) {
-            cmp = compareFileNames(files[j], files[j+1]);
-            if (cmp < 0) {
-                temp = files[j + 1];
-                files[j + 1] = files[j];
-                files[j] = temp;
-            }
-        }
-    }
-
-    return TRUE;
-}
-
-int sortFilesNamesDec(TCHAR **files, int cnt) {
-    int i, j;
-    TCHAR *temp;
-    int cmp;
-
-    for (i = 0; i < cnt; i++) {
-        for (j = 0; j < cnt - 1; j++) {
-            cmp = compareFileNames(files[j], files[j+1]);
-            if (cmp > 0) {
-                temp = files[j + 1];
-                files[j + 1] = files[j];
-                files[j] = temp;
-            }
-        }
-    }
-
-    return TRUE;
-}
-
-/**
- * Returns a NULL terminated list of file names within the specified pattern.
- *  The files will be sorted new to old for TIMES.  Then incremental ordering
- *  for NAMES.  The numeric components of the names will be treated as
- *  numbers and sorted accordingly.
- */
-TCHAR** wrapperFileGetFiles(const TCHAR* pattern, int sortMode) {
-    int cnt;
-    int filesSize;
-    TCHAR **files;
-#ifdef WIN32
-    int i;
-    size_t dirLen;
-    TCHAR *c;
-    TCHAR *dirPart;
-    intptr_t handle;
-    struct _tfinddata64_t fblock;
-    size_t fileLen;
-    TCHAR **newFiles;
-    __time64_t *fileTimes;
-    __time64_t *newFileTimes;
-#else
-#ifdef WRAPPER_FILE_DEBUG
-    int i;
-#endif
-    int result;
-    glob_t g;
-    int findex;
-    time_t *fileTimes;
-    struct stat fileStat;
-#endif
-
-#ifdef WRAPPER_FILE_DEBUG
-    _tprintf(TEXT("wrapperFileGetFiles(%s, %d)\n"), pattern, sortMode);
-#endif
-
-#ifdef WIN32
-    cnt = 0;
-    /* Initialize the files array. */
-    filesSize = FILES_CHUNK;
-    files = malloc(sizeof(TCHAR *) * filesSize);
-    if (!files) {
-        outOfMemoryQueued(TEXT("WFGF"), 1);
-        return NULL;
-    }
-    memset(files, 0, sizeof(TCHAR *) * filesSize);
-
-    fileTimes = malloc(sizeof(__time64_t) * filesSize);
-    if (!fileTimes) {
-        outOfMemoryQueued(TEXT("WFGF"), 2);
-        free(files);
-        return NULL;
-    }
-    memset(fileTimes, 0, sizeof(__time64_t) * filesSize);
-
-    /* Extract any path information from the beginning of the file */
-    c = max(_tcsrchr(pattern, TEXT('\\')), _tcsrchr(pattern, TEXT('/')));
-    if (c == NULL) {
-        /* No directory component */
-        dirPart = malloc(sizeof(TCHAR) * 1);
-        if (!dirPart) {
-            outOfMemoryQueued(TEXT("WFGF"), 3);
-            return NULL;
-        }
-        dirPart[0] = TEXT('\0');
-        dirLen = 0;
-    } else {
-        /* extract the directory. */
-        dirLen = c - pattern + 1;
-        dirPart = malloc(sizeof(TCHAR) * (dirLen + 1));
-        if (!dirPart) {
-            outOfMemoryQueued(TEXT("WFGF"), 4);
-            return NULL;
-        }
-        _tcsncpy(dirPart, pattern, dirLen);
-        dirPart[dirLen] = TEXT('\0');
-    }
-
-#ifdef WRAPPER_FILE_DEBUG
-    _tprintf(TEXT("  dirPart=[%s]\n"), dirPart);
-#endif
-
-    /* Get the first file. */
-    if ((handle = _tfindfirst64(pattern, &fblock)) > 0) {
-        if ((_tcscmp(fblock.name, TEXT(".")) != 0) && (_tcscmp(fblock.name, TEXT("..")) != 0)) {
-            fileLen = _tcslen(fblock.name);
-            files[cnt] = malloc((_tcslen(dirPart) + _tcslen(fblock.name) + 1) * sizeof(TCHAR));
-            if (!files[cnt]) {
-                outOfMemoryQueued(TEXT("WFGF"), 5);
-                free(fileTimes);
-                wrapperFileFreeFiles(files);
-                free(dirPart);
-                return NULL;
-            }
-            _sntprintf(files[cnt], _tcslen(dirPart) + _tcslen(fblock.name) + 1, TEXT("%s%s"), dirPart, fblock.name);
-            fileTimes[cnt] = fblock.time_write;
-#ifdef WRAPPER_FILE_DEBUG
-            _tprintf(TEXT("  files[%d]=%s, %ld\n"), cnt, files[cnt], fileTimes[cnt]);
-#endif
-
-            cnt++;
-        }
-
-        /* Look for additional files. */
-        while (_tfindnext64(handle, &fblock) == 0) {
-            if ((_tcscmp(fblock.name, TEXT(".")) != 0) && (_tcscmp(fblock.name, TEXT("..")) != 0)) {
-                /* Make sure we have enough room in the files array. */
-                if (cnt >= filesSize - 1) {
-                    newFiles = malloc(sizeof(TCHAR *) * (filesSize + FILES_CHUNK));
-                    if (!newFiles) {
-                        outOfMemoryQueued(TEXT("WFGF"), 6);
-                        free(fileTimes);
-                        wrapperFileFreeFiles(files);
-                        free(dirPart);
-                        return NULL;
-                    }
-                    memset(newFiles, 0, sizeof(TCHAR *) * (filesSize + FILES_CHUNK));
-                    newFileTimes = malloc(sizeof(__time64_t) * (filesSize + FILES_CHUNK));
-                    if (!newFileTimes) {
-                        outOfMemoryQueued(TEXT("WFGF"), 7);
-                        free(newFiles);
-                        free(fileTimes);
-                        wrapperFileFreeFiles(files);
-                        free(dirPart);
-                        return NULL;
-                    }
-                    memset(newFileTimes, 0, sizeof(__time64_t) * (filesSize + FILES_CHUNK));
-                    
-                    for (i = 0; i < filesSize; i++) {
-                        newFiles[i] = files[i];
-                        newFileTimes[i] = fileTimes[i];
-                    }
-                    free(files);
-                    free(fileTimes);
-                    files = newFiles;
-                    fileTimes = newFileTimes;
-                    filesSize += FILES_CHUNK;
-#ifdef WRAPPER_FILE_DEBUG
-                    _tprintf(TEXT("  increased files to %d\n"), filesSize);
-#endif
-                }
-
-                fileLen = _tcslen(fblock.name);
-                files[cnt] = malloc((_tcslen(dirPart) + _tcslen(fblock.name) + 1) * sizeof(TCHAR));
-                if (!files[cnt]) {
-                    outOfMemoryQueued(TEXT("WFGF"), 8);
-                    free(fileTimes);
-                    wrapperFileFreeFiles(files);
-                    free(dirPart);
-                    return NULL;
-                }
-                _sntprintf(files[cnt], _tcslen(dirPart) + _tcslen(fblock.name) + 1, TEXT("%s%s"), dirPart, fblock.name);
-                fileTimes[cnt] = fblock.time_write;
-
-#ifdef WRAPPER_FILE_DEBUG
-                _tprintf(TEXT("  files[%d]=%s, %ld\n"), cnt, files[cnt], fileTimes[cnt]);
-#endif
-                cnt++;
-            }
-        }
-
-        /* Close the file search */
-        _findclose(handle);
-    }
-
-    if (cnt <= 0) {
-        if (errno == ENOENT) {
-            /* No files matched. */
-#ifdef WRAPPER_FILE_DEBUG
-            _tprintf(TEXT("  No files matched.\n"));
-#endif
-        } else {
-            /* Encountered an error of some kind. */
-            log_printf_queue(TRUE, WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR, TEXT("Error listing files, %s: %s"), pattern, getLastErrorText());
-            free(fileTimes);
-            wrapperFileFreeFiles(files);
-            return NULL;
-        }
-    }
-#else
-
-#ifdef UNICODE
-    char* cPattern;
-    size_t req;
-
-    req = wcstombs(NULL, pattern, 0) + 1;
-    cPattern = malloc(req);
-    if(!cPattern) {
-        outOfMemoryQueued(TEXT("WFGF"), 8);
-        return NULL;
-    }
-    wcstombs(cPattern, pattern, req);
-
-    result = glob(cPattern, GLOB_MARK | GLOB_NOSORT, NULL, &g);
-    free(cPattern);
-#else
-    result = glob(pattern, GLOB_MARK | GLOB_NOSORT, NULL, &g);
-#endif
-    cnt = 0;
-    if (!result) {
-        if (g.gl_pathc > 0) {
-            filesSize = g.gl_pathc + 1;
-            files = malloc(sizeof(TCHAR *) * filesSize);
-            if (!files) {
-                outOfMemoryQueued(TEXT("WFGF"), 9);
-                return NULL;
-            }
-            memset(files, 0, sizeof(TCHAR *) * filesSize);
-            
-            fileTimes = malloc(sizeof(time_t) * filesSize);
-            if (!fileTimes) {
-                outOfMemoryQueued(TEXT("WFGF"), 10);
-                wrapperFileFreeFiles(files);
-                return NULL;
-            }
-            memset(fileTimes, 0, sizeof(time_t) * filesSize);
-
-            for (findex = 0; findex < g.gl_pathc; findex++) {
-#ifdef UNICODE
-                req = mbstowcs(NULL, g.gl_pathv[findex], 0);
-                if (req == (size_t)-1) {
-                    invalidMultiByteSequence(TEXT("GLET"), 1);
-                }
-                files[cnt] = malloc((req + 1) * sizeof(TCHAR));
-                if (!files[cnt]) {
-                    outOfMemoryQueued(TEXT("WFGF"), 11);
-                    free(fileTimes);
-                    wrapperFileFreeFiles(files);
-                    return NULL;
-                }
-                mbstowcs(files[cnt], g.gl_pathv[findex], req + 1);
-
-#else
-                files[cnt] = malloc((strlen(g.gl_pathv[findex]) + 1));
-                if (!files[cnt]) {
-                    outOfMemoryQueued(TEXT("WFGF"), 11);
-                    free(fileTimes);
-                    wrapperFileFreeFiles(files);
-                    return NULL;
-                }
-
-                strncpy(files[cnt], g.gl_pathv[findex], strlen(g.gl_pathv[findex]) + 1);
-#endif
-
-                /* Only try to get the modified time if it is really necessary. */
-                if (sortMode == WRAPPER_FILE_SORT_MODE_TIMES) {
-                    if (!_tstat(files[cnt], &fileStat)) {
-                        fileTimes[cnt] = fileStat.st_mtime;
-                    } else {
-                        log_printf_queue(TRUE, WRAPPER_SOURCE_WRAPPER, LEVEL_WARN, TEXT("Failed to stat %s: %s"), files[cnt], getLastErrorText());
-                    }
-                }
-#ifdef WRAPPER_FILE_DEBUG
-                printf("  files[%d]=%s, %ld\n", cnt, files[cnt], fileTimes[cnt]);
-#endif
-                cnt++;
-            }
-        } else {
-#ifdef WRAPPER_FILE_DEBUG
-            printf("  No files matched.\n");
-#endif
-            /* No files, but we still need the array. */
-            filesSize = 1;
-            files = malloc(sizeof(TCHAR *) * filesSize);
-            if (!files) {
-                outOfMemoryQueued(TEXT("WFGF"), 12);
-                return NULL;
-            }
-            memset(files, 0, sizeof(TCHAR *) * filesSize);
-            
-            fileTimes = malloc(sizeof(time_t) * filesSize);
-            if (!fileTimes) {
-                free(files);
-                outOfMemoryQueued(TEXT("WFGF"), 13);
-                return NULL;
-            }
-            memset(fileTimes, 0, sizeof(time_t) * filesSize);
-        }
-
-        globfree(&g);
-    } else if (result == GLOB_NOMATCH) {
-#ifdef WRAPPER_FILE_DEBUG
-        _tprintf(TEXT("  No files matched.\n"));
-#endif
-        /* No files, but we still need the array. */
-        filesSize = 1;
-        files = malloc(sizeof(TCHAR *) * filesSize);
-        if (!files) {
-            outOfMemoryQueued(TEXT("WFGF"), 14);
-            return NULL;
-        }
-        memset(files, 0, sizeof(TCHAR *) * filesSize);
-
-        fileTimes = malloc(sizeof(time_t) * filesSize);
-        if (!fileTimes) {
-            free(files);
-            outOfMemoryQueued(TEXT("WFGF"), 15);
-            return NULL;
-        }
-        memset(fileTimes, 0, sizeof(time_t) * filesSize);
-    } else {
-        /* Encountered an error of some kind. */
-        log_printf_queue(TRUE, WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR, TEXT("Error listing files, %s: %s"), pattern, getLastErrorText());
-        return NULL;
-    }
-#endif
-    
-    if (sortMode == WRAPPER_FILE_SORT_MODE_TIMES) {
-        if (!sortFilesTimes(files, fileTimes, cnt)) {
-            /* Failed. Reported. */
-            free(fileTimes);
-            wrapperFileFreeFiles(files);
-            return NULL;
-        }
-    } else if (sortMode == WRAPPER_FILE_SORT_MODE_NAMES_DEC) {
-        if (!sortFilesNamesDec(files, cnt)) {
-            /* Failed. Reported. */
-            free(fileTimes);
-            wrapperFileFreeFiles(files);
-            return NULL;
-        }
-    } else { /* WRAPPER_FILE_SORT_MODE_NAMES_ASC */
-        if (!sortFilesNamesAsc(files, cnt)) {
-            /* Failed. Reported. */
-            free(fileTimes);
-            wrapperFileFreeFiles(files);
-            return NULL;
-        }
-    }
-
-#ifdef WRAPPER_FILE_DEBUG
-    _tprintf(TEXT("  Sorted:\n"));
-    for (i = 0; i < cnt; i++) {
-        _tprintf(TEXT("  files[%d]=%s, %ld\n"), i, files[i], fileTimes[i]);
-    }
-    _tprintf(TEXT("wrapperFileGetFiles(%s, %d) END\n"), pattern, sortMode);
-#endif
-
-    free(fileTimes);
-
-    return files;
-}
-
-/**
- * Frees the array of file names returned by wrapperFileGetFiles()
- */
-void wrapperFileFreeFiles(TCHAR** files) {
-    int i;
-
-    i = 0;
-    while (files[i]) {
-        free(files[i]);
-        i++;
-    }
-    free(files);
-}
+/* Structure used by configFileReader to read files. */
+typedef struct ConfigFileReader ConfigFileReader;
+struct ConfigFileReader {
+    ConfigFileReader_Callback callback;
+    void *callbackParam;
+    int enableIncludes;
+    int preload;
+    /* debugIncludes controls whether or not debug output is logged.  It is set using directives in the file being read. */
+    int debugIncludes;
+    /* debugProperties controls whether or not debug output is logged.  It is set using directives in the file being read. */
+    int debugProperties;
+};
 
 /**
  * Tests whether a file exists.
@@ -610,6 +75,7 @@ int wrapperFileExists(const TCHAR * filename) {
     return FALSE;
 }
 
+#ifdef WIN32
 /**
  * @param path to check.
  * @param advice 0 if advice should be displayed.
@@ -617,7 +83,6 @@ int wrapperFileExists(const TCHAR * filename) {
  * @return advice or advice + 1 if advice was logged.
  */
 int wrapperGetUNCFilePath(const TCHAR *path, int advice) {
-#ifdef WIN32
     TCHAR drive[4];
     DWORD result;
 
@@ -639,62 +104,10 @@ int wrapperGetUNCFilePath(const TCHAR *path, int advice) {
             }
         }
     }
-#endif
     return advice;
 }
-
-#ifdef WRAPPER_FILE_DEBUG
-void wrapperFileTests() {
-    TCHAR** files;
-
-    printf("Start wrapperFileTests\n");
-    files = wrapperFileGetFiles((TEXT("../logs/*.log*"), WRAPPER_FILE_SORT_MODE_TIMES);
-    if (files) {
-        wrapperFileFreeFiles(files);
-    }
-
-    files = wrapperFileGetFiles(TEXT("../logs/*.log*"), WRAPPER_FILE_SORT_MODE_NAMES_ASC);
-    if (files) {
-        wrapperFileFreeFiles(files);
-    }
-
-    files = wrapperFileGetFiles(TEXT("../logs/*.log*"), WRAPPER_FILE_SORT_MODE_NAMES_DEC);
-    if (files) {
-        wrapperFileFreeFiles(files);
-    }
-    printf("End wrapperFileTests\n");
-}
 #endif
 
-
-/**
- * Call functions in property.c temporarily.
- */
-extern void evaluateEnvironmentVariables(const TCHAR *propertyValue, TCHAR *buffer, int bufferLength, int warnUndefinedVars, void *warnedUndefVarMap, int warnLogLevel);
-#ifdef WIN32
-#define strIgnoreCaseCmp _stricmp
-extern int getEncodingByName(char* encodingMB, int *encoding);
-#else
-#define strIgnoreCaseCmp strcasecmp
-extern int getEncodingByName(char* encodingMB, char** encoding);
-#endif
-
-/**
- * Initialize `reader'
- */
-void configFileReader_Initialize(ConfigFileReader *reader,
-                 ConfigFileReader_Callback callback,
-                 void *callbackParam,
-                 int enableIncludes)
-{
-    reader->callback = callback;
-    reader->callbackParam = callbackParam;
-    reader->enableIncludes = enableIncludes;
-    reader->debugIncludes = FALSE;
-    reader->debugProperties = FALSE;
-    reader->preload = FALSE;
-}
-
 /**
  * Read configuration file.
  */
@@ -703,7 +116,13 @@ int configFileReader_Read(ConfigFileReader *reader,
                           int fileRequired,
                           int depth,
                           const TCHAR *parentFilename,
-                          int parentLineNumber)
+                          int parentLineNumber,
+                          const TCHAR *argCommand,
+                          const TCHAR *originalWorkingDir,
+                          PHashMap warnedVarMap,
+                          int logWarnings,
+                          int logWarningLogLevel,
+                          int isDebugging)
 {
     FILE *stream;
     char bufferMB[MAX_PROPERTY_NAME_VALUE_LENGTH];
@@ -746,10 +165,10 @@ int configFileReader_Read(ConfigFileReader *reader,
             if (depth > 0) {
                 log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_STATUS,
                     TEXT("%sIncluded configuration file not found: %s\n  Referenced from: %s (line %d)\n  Current working directory: %s"),
-                    (reader->debugIncludes ? TEXT("  ") : TEXT("")), filename, parentFilename, parentLineNumber, wrapperData->originalWorkingDir);
+                    (reader->debugIncludes ? TEXT("  ") : TEXT("")), filename, parentFilename, parentLineNumber, originalWorkingDir);
             } else {
                 log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_STATUS,
-                    TEXT("Configuration file not found: %s\n  Current working directory: %s"), filename, wrapperData->originalWorkingDir);
+                    TEXT("Configuration file not found: %s\n  Current working directory: %s"), filename, originalWorkingDir);
             }
         } else {
 #ifdef _DEBUG
@@ -987,7 +406,7 @@ int configFileReader_Read(ConfigFileReader *reader,
                         log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_STATUS,
                             TEXT("Found #include file in %s: %s"), filename, c);
                     }
-                    evaluateEnvironmentVariables(c, expBuffer, MAX_PROPERTY_NAME_VALUE_LENGTH, FALSE, NULL, LEVEL_WARN);
+                    evaluateEnvironmentVariables(c, expBuffer, MAX_PROPERTY_NAME_VALUE_LENGTH, logWarnings, warnedVarMap, logWarningLogLevel);
 
                     if (reader->debugIncludes && (_tcscmp(c, expBuffer) != 0)) {
                         /* Only show this log if there were any environment variables. */
@@ -1003,7 +422,7 @@ int configFileReader_Read(ConfigFileReader *reader,
                         if (reader->debugIncludes || includeRequired) {
                             log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_STATUS,
                                 TEXT("Unable to resolve the full path of included configuration file: %s (%s)\n  Referenced from: %s (line %d)\n  Current working directory: %s"),
-                                expBuffer, getLastErrorText(), filename, lineNumber, wrapperData->originalWorkingDir);
+                                expBuffer, getLastErrorText(), filename, lineNumber, originalWorkingDir);
                         }
                         absoluteBuffer = NULL;
                     } else {
@@ -1015,7 +434,7 @@ int configFileReader_Read(ConfigFileReader *reader,
                                 if (reader->debugIncludes || includeRequired) {
                                     log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_STATUS,
                                         TEXT("Unable to resolve the full path of included configuration file: %s (%s)\n  Referenced from: %s (line %d)\n  Current working directory: %s"),
-                                        expBuffer, getLastErrorText(), filename, lineNumber, wrapperData->originalWorkingDir);
+                                        expBuffer, getLastErrorText(), filename, lineNumber, originalWorkingDir);
                                 }
                                 free(absoluteBuffer);
                                 absoluteBuffer = NULL;
@@ -1027,11 +446,11 @@ int configFileReader_Read(ConfigFileReader *reader,
                     if (!absoluteBuffer) {
                         outOfMemory(TEXT("RCF"), 2);
                     } else {
-                        if (_trealpath(expBuffer, absoluteBuffer) == NULL) {
+                        if (_trealpathN(expBuffer, absoluteBuffer, PATH_MAX + 1) == NULL) {
                             if (reader->debugIncludes || includeRequired) {
                                 log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_STATUS,
                                     TEXT("Unable to resolve the full path of included configuration file: %s (%s)\n  Referenced from: %s (line %d)\n  Current working directory: %s"),
-                                    expBuffer, getLastErrorText(), filename, lineNumber, wrapperData->originalWorkingDir);
+                                    expBuffer, getLastErrorText(), filename, lineNumber, originalWorkingDir);
                             }
                             free(absoluteBuffer);
                             absoluteBuffer = NULL;
@@ -1040,7 +459,7 @@ int configFileReader_Read(ConfigFileReader *reader,
 #endif
                     if (absoluteBuffer) {
                         if (depth < MAX_INCLUDE_DEPTH) {
-                            readResult = configFileReader_Read(reader, absoluteBuffer, includeRequired, depth + 1, filename, lineNumber);
+                            readResult = configFileReader_Read(reader, absoluteBuffer, includeRequired, depth + 1, filename, lineNumber, argCommand, originalWorkingDir, warnedVarMap, logWarnings, logWarningLogLevel, isDebugging);
                             if (readResult == CONFIG_FILE_READER_SUCCESS) {
                                 /* Ok continue. */
                             } else if ((readResult == CONFIG_FILE_READER_FAIL) || (readResult == CONFIG_FILE_READER_HARD_FAIL)) {
@@ -1112,4 +531,52 @@ int configFileReader_Read(ConfigFileReader *reader,
     return readResult;
 }
 
+/**
+ * Reads configuration lines from the file `filename' and calls `callback' with the line and
+ *  `callbackParam' specified to its arguments.
+ *
+ * @param filename Name of configuration file to read.
+ * @param fileRequired TRUE if the file specified by filename is required, FALSE if a missing
+ *                     file will silently fail.
+ * @param callback Pointer to a callback funtion which will be called for each line read.
+ * @param callbackParam Pointer to additional user data which will be passed to the callback.
+ * @param enableIncludes If TRUE then includes will be supported.
+ * @param preload TRUE if this is being called in the preload step meaning that all errors
+ *                should be suppressed.
+ * @param argCommand Argument passed to the binary.
+ * @param originalWorkingDir Working directory of the binary at the moment it was launched.
+ * @param warnedVarMap Map of undefined environment variables for which the user was warned.
+ * @param logWarnings Flag that controls whether or not warnings will be logged.
+ * @param logWarningLogLevel Log level at which any log warnings will be logged.
+ * @param isDebugging Flag that controls whether or not debug output will be logged.
+ *
+ * @return CONFIG_FILE_READER_SUCCESS if the file was read successfully,
+ *         CONFIG_FILE_READER_FAIL if there were any problems at all, or
+ *         CONFIG_FILE_READER_HARD_FAIL if the problem should cascaded all the way up.
+ */
+int configFileReader(const TCHAR *filename,
+                     int fileRequired,
+                     ConfigFileReader_Callback callback,
+                     void *callbackParam,
+                     int enableIncludes,
+                     int preload,
+                     const TCHAR *argCommand,
+                     const TCHAR *originalWorkingDir,
+                     PHashMap warnedVarMap,
+                     int logWarnings,
+                     int logWarningLogLevel,
+                     int isDebugging) {
+    ConfigFileReader reader;
+    
+    /* Initialize the reader. */
+    reader.callback = callback;
+    reader.callbackParam = callbackParam;
+    reader.enableIncludes = enableIncludes;
+    reader.preload = preload;
+    reader.debugIncludes = FALSE;
+    reader.debugProperties = FALSE;
+    
+    return configFileReader_Read(&reader, filename, fileRequired, 0, NULL, 0, argCommand, originalWorkingDir, warnedVarMap, logWarnings, logWarningLogLevel, isDebugging);
+}
+
 
