@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2013 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -35,6 +35,7 @@
 
 #ifndef WIN32
 
+#include <limits.h>
 #include <wchar.h>
 #include <netdb.h>
 #include <stdio.h>
@@ -63,11 +64,11 @@
 #include <sys/time.h>
 
 #if defined(IRIX)
-#define PATH_MAX FILENAME_MAX
+ #define PATH_MAX FILENAME_MAX
 #endif
 
 #ifndef USE_USLEEP
-#include <time.h>
+ #include <time.h>
 #endif
 
 #ifndef getsid
@@ -85,6 +86,12 @@ int pipedes[2] = {-1, -1};
 #define PIPE_READ_END 0
 #define PIPE_WRITE_END 1
 
+/**
+ * maximum length for a user name should be 8, 
+ * but according to 'man useradd' it may be 32
+ */
+#define MAX_USER_NAME_LENGTH 32
+
 TCHAR wrapperClasspathSeparator = TEXT(':');
 
 int javaIOThreadSet = FALSE;
@@ -155,13 +162,6 @@ void appExit(int exitCode, int argc, TCHAR** argv) {
     exit(exitCode);
 }
 
-/**
- * Gets the error code for the last operation that failed.
- */
-int wrapperGetLastError() {
-    return errno;
-}
-
 /**
  * Writes a PID to disk.
  *
@@ -222,7 +222,7 @@ void takeSignalAction(int sigNum, const TCHAR *sigName, int mode) {
         switch (mode) {
         case WRAPPER_SIGNAL_MODE_RESTART:
             log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_STATUS,
-                TEXT("%s trapped.  Restarting JVM."), sigName);
+                TEXT("%s trapped.  %s"), sigName, wrapperGetRestartProcessMessage());
             wrapperRestartProcess();
             break;
 
@@ -432,7 +432,7 @@ void descSignal(siginfo_t *sigInfo) {
  #ifdef UNICODE
     size_t req;
  #endif
-    TCHAR *uName;
+    TCHAR uName[MAX_USER_NAME_LENGTH + 1];
 #endif
 
     /* Not supported on all platforms */
@@ -466,23 +466,25 @@ void descSignal(siginfo_t *sigInfo) {
                 sigInfo->si_errno,
                 strerror(sigInfo->si_errno));
         }
-
+            
 #ifdef SI_USER
         if (sigInfo->si_code == SI_USER) {
             pw = getpwuid(sigInfo->si_uid);
             if (pw == NULL) {
-                uName = TEXT("<unknown>");
+                _sntprintf(uName, MAX_USER_NAME_LENGTH + 1, TEXT("<unknown>"));
             } else {
  #ifndef UNICODE
-                uName = pw->pw_name;
+                _sntprintf(uName, MAX_USER_NAME_LENGTH + 1, TEXT("%s"), pw->pw_name);
  #else
-                req = mbstowcs(NULL, pw->pw_name, 0) + 1;
-                uName = malloc(req * sizeof(TCHAR));
-                if (!uName) {
-                    outOfMemory(TEXT("DSCS"), 1);
+                req = mbstowcs(NULL, pw->pw_name, MBSTOWCS_QUERY_LENGTH);
+                if (req == (size_t)-1) {
                     return;
                 }
+                if (req > MAX_USER_NAME_LENGTH) {
+                    req = MAX_USER_NAME_LENGTH;
+                }
                 mbstowcs(uName, pw->pw_name, req + 1);
+                uName[req] = TEXT('\0'); /* Avoid bufferflows caused by badly encoded characters. */
  #endif
             }
 
@@ -494,9 +496,6 @@ void descSignal(siginfo_t *sigInfo) {
                 TEXT("  signal generated by PID: %d (Session PID: %d), UID: %d (%s)"),
  #endif
                 sigInfo->si_pid, getsid(sigInfo->si_pid), sigInfo->si_uid, uName);
- #ifdef UNICODE
-            free(uName);
- #endif
         }
 #endif
     }
@@ -1066,6 +1065,17 @@ int wrapperBuildJavaCommand() {
     int length, i;
 
     /* If this is not the first time through, then dispose the old command array */
+    if (wrapperData->jvmVersionCommand) {
+        i = 0;
+        while(wrapperData->jvmVersionCommand[i] != NULL) {
+            free(wrapperData->jvmVersionCommand[i]);
+            wrapperData->jvmVersionCommand[i] = NULL;
+            i++;
+        }
+
+        free(wrapperData->jvmVersionCommand);
+        wrapperData->jvmVersionCommand = NULL;
+    }
     if (wrapperData->jvmCommand) {
         i = 0;
         while(wrapperData->jvmCommand[i] != NULL) {
@@ -1094,28 +1104,37 @@ int wrapperBuildJavaCommand() {
         return TRUE;
     }
 
-    if (wrapperData->commandLogLevel != LEVEL_NONE) {
-        for (i = 0; i < length; i++) {
-            log_printf(WRAPPER_SOURCE_WRAPPER, wrapperData->commandLogLevel,
-                TEXT("Command[%d] : %s"), i, strings[i]);
-        }
-
-        if (wrapperData->environmentClasspath) {
-            log_printf(WRAPPER_SOURCE_WRAPPER, wrapperData->commandLogLevel, TEXT(
-                "Classpath in Environment : %s"), wrapperData->classpath);
-        }
+    /* Allocate memory to hold array of version command strings.  The array is itself NULL terminated */
+    wrapperData->jvmVersionCommand = malloc(sizeof(TCHAR *) * (2 + 1));
+    if (!wrapperData->jvmVersionCommand) {
+        outOfMemory(TEXT("WBJC"), 1);
+        return TRUE;
     }
-
-    if (wrapperData->environmentClasspath) {
-        setEnv(TEXT("CLASSPATH"), wrapperData->classpath, ENV_SOURCE_WRAPPER);
+    memset(wrapperData->jvmVersionCommand, 0, sizeof(TCHAR *) * (2 + 1));
+    /* Java Command */
+    wrapperData->jvmVersionCommand[0] = malloc(sizeof(TCHAR) * (_tcslen(strings[0]) + 1));
+    if (!wrapperData->jvmVersionCommand[0]) {
+        outOfMemory(TEXT("WBJC"), 2);
+        return TRUE;
     }
-
+    _tcsncpy(wrapperData->jvmVersionCommand[0], strings[0], _tcslen(strings[0]) + 1);
+    /* -version */
+    wrapperData->jvmVersionCommand[1] = malloc(sizeof(TCHAR) * (8 + 1));
+    if (!wrapperData->jvmVersionCommand[1]) {
+        outOfMemory(TEXT("WBJC"), 3);
+        return TRUE;
+    }
+    _tcsncpy(wrapperData->jvmVersionCommand[1], TEXT("-version"), 8 + 1);
+    /* NULL */
+    wrapperData->jvmVersionCommand[2] = NULL;
+    
     /* Allocate memory to hold array of command strings.  The array is itself NULL terminated */
     wrapperData->jvmCommand = malloc(sizeof(TCHAR *) * (length + 1));
     if (!wrapperData->jvmCommand) {
         outOfMemory(TEXT("WBJC"), 1);
         return TRUE;
     }
+    memset(wrapperData->jvmCommand, 0, sizeof(TCHAR *) * (length + 1));
     /* number of arguments + 1 for a NULL pointer at the end */
     for (i = 0; i <= length; i++) {
         if (i < length) {
@@ -1125,6 +1144,8 @@ int wrapperBuildJavaCommand() {
                 return TRUE;
             }
             _tcsncpy(wrapperData->jvmCommand[i], strings[i], _tcslen(strings[i]) + 1);
+            
+            wrapperData->jvmCommand[i] = wrapperPostProcessCommandElement(wrapperData->jvmCommand[i]);
         } else {
             wrapperData->jvmCommand[i] = NULL;
         }
@@ -1137,16 +1158,83 @@ int wrapperBuildJavaCommand() {
 }
 
 /**
- * Launches a JVM process and stores it internally.
+ * Calculate the total length of the environment assuming that they are separated by spaces.
  */
-void wrapperExecute() {
+size_t wrapperCalculateEnvironmentLength() {
+    /* The compiler won't let us reverence environ directly in the for loop on OSX because it is actually a function. */
+    char **environment = environ;
+    size_t i;
+    size_t len;
+    size_t lenTotal;
+    
+    i = 0;
+    lenTotal = 0;
+    while (environment[i]) {
+        /* All we need is the length so we don't actually have to convert them. */
+        len = mbstowcs(NULL, environment[i], MBSTOWCS_QUERY_LENGTH);
+        if (len == (size_t)-1) {
+            /* Invalid string.  Skip. */
+        } else {
+            /* Add length of variable + null + pointer to next element */
+            lenTotal += len + 1 + sizeof(char *);
+        }
+        i++;
+    }
+    /* Null termination of the list. */
+    lenTotal += sizeof(char *) + sizeof(char *);
+    
+    return lenTotal;
+}
+
+/**
+ * Launches a JVM process and stores it internally.
+ *
+ * @return TRUE if there were any problems.  When this happens the Wrapper will not try to restart.
+ */
+int wrapperExecute() {
+    int i;
     pid_t proc;
+    int execErrno;
+    size_t lenCmd;
+    size_t lenEnv;
 
     /* Create the pipe. */
     if (pipe(pipedes) < 0) {
         log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR,
                    TEXT("Could not init pipe: %s"), getLastErrorText());
-        return;
+        return TRUE;
+    }
+    
+    /* Log the Java commands. */
+    
+    /* If the JVM version printout is requested then log its command line first. */
+    if (wrapperData->printJVMVersion) {
+        if (wrapperData->isDebugging) {
+            log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_DEBUG, TEXT("Java Command Line (Query Java Version):"));
+            for (i = 0; wrapperData->jvmVersionCommand[i] != NULL; i++) {
+                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_DEBUG,
+                    TEXT("  Command[%d] : %s"), i, wrapperData->jvmVersionCommand[i]);
+            }
+        }
+    }
+    
+    /* Log ghe application java command line */
+    if (wrapperData->commandLogLevel != LEVEL_NONE) {
+        log_printf(WRAPPER_SOURCE_WRAPPER, wrapperData->commandLogLevel, TEXT("Java Command Line:"));
+        for (i = 0; wrapperData->jvmCommand[i] != NULL; i++) {
+            log_printf(WRAPPER_SOURCE_WRAPPER, wrapperData->commandLogLevel,
+                TEXT("  Command[%d] : %s"), i, wrapperData->jvmCommand[i]);
+        }
+
+        if (wrapperData->environmentClasspath) {
+            log_printf(WRAPPER_SOURCE_WRAPPER, wrapperData->commandLogLevel,
+                TEXT("  Classpath in Environment : %s"), wrapperData->classpath);
+        }
+    }
+
+    /* Update the CLASSPATH in the environment if requested so the JVM can access it. */ 
+    if (wrapperData->environmentClasspath) {
+        setEnv(TEXT("CLASSPATH"), wrapperData->classpath, ENV_SOURCE_APPLICATION);
     }
 
     /* Make sure the log file is closed before the Java process is created.  Failure to do
@@ -1173,7 +1261,7 @@ void wrapperExecute() {
                    TEXT("Could not spawn JVM process: %s"), getLastErrorText());
 
         /* The pipedes array is global so do not close the pipes. */
-
+        return TRUE;
     } else {
         /* Reset the exit code when we launch a new JVM. */
         wrapperData->exitCode = 0;
@@ -1197,14 +1285,18 @@ void wrapperExecute() {
             if (dup2(pipedes[PIPE_WRITE_END], STDOUT_FILENO) < 0) {
                 log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR,
                     TEXT("%sUnable to set JVM's stdout: %s"), LOG_FORK_MARKER, getLastErrorText());
-                return;
+                /* This process needs to end. */
+                exit(1);
+                return TRUE; /* Will not get here. */
             }
 
             /* Send errors to the pipe by dupicating the pipe fd and setting the copy as the stderr fd. */
             if (dup2(pipedes[PIPE_WRITE_END], STDERR_FILENO) < 0) {
                 log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR,
                     TEXT("%sUnable to set JVM's stderr: %s"), LOG_FORK_MARKER, getLastErrorText());
-                return;
+                /* This process needs to end. */
+                exit(1);
+                return TRUE; /* Will not get here. */
             }
             
             /* Close both ends of the pipe as we have already duplicated the Write end for our purposes. */
@@ -1229,10 +1321,26 @@ void wrapperExecute() {
             /* The pipedes array is global so do not close the pipes. */
             /* Child process: execute the JVM. */
             _texecvp(wrapperData->jvmCommand[0], wrapperData->jvmCommand);
+            execErrno = errno;
 
             /* We reached this point...meaning we were unable to start. */
             log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR,
-                TEXT("%sUnable to start JVM: %s (%d)"), LOG_FORK_MARKER, getLastErrorText(), errno);
+                TEXT("%sUnable to start JVM: %s (%d)"), LOG_FORK_MARKER, getLastErrorText(), execErrno);
+            if (execErrno == E2BIG) {
+                /* Command line too long. */
+                /* Calculate the total length of the command line. */
+                lenCmd = 0;
+                for (i = 0; wrapperData->jvmCommand[i] != NULL; i++) {
+                    lenCmd += _tcslen(wrapperData->jvmCommand[i]) + 1;
+                }
+                lenEnv = wrapperCalculateEnvironmentLength();
+                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR, TEXT("%s  The generated command line plus the environment was larger than the maximum allowed."), LOG_FORK_MARKER);
+                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR, TEXT("%s  The current length is %d bytes of which %d is the command line, and %d is the environment."), LOG_FORK_MARKER, lenCmd + lenEnv + 1, lenCmd, lenEnv); 
+                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR, TEXT("%s  It is not possible to calculate an exact maximum length as it depends on a number of factors for each system."), LOG_FORK_MARKER);
+
+                /* TODO: Figure out a way to inform the Wrapper not to restart and try again as repeatedly doing this is meaningless. */
+            }
+
 
             if (wrapperData->isAdviserEnabled) {
                 log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ADVICE, TEXT("%s"), LOG_FORK_MARKER );
@@ -1257,8 +1365,9 @@ void wrapperExecute() {
 
             /* This process needs to end. */
             exit(1);
+            return TRUE; /* Will not get here. */
         } else {
-            /* We are the parent side. */
+            /* We are the parent side and need to assume that at this point the JVM is up. */
             wrapperData->javaPID = proc;
             
             /* Close the write end as it is not used. */
@@ -1298,6 +1407,8 @@ void wrapperExecute() {
                         TEXT("Unable to write the Java Id file: %s"), wrapperData->javaIdFilename);
                 }
             }
+            
+            return FALSE;
         }
     }
 }
@@ -1665,7 +1776,7 @@ int setWorkingDir(TCHAR *app) {
     }
 
     /* Set a variable to the location of the binary. */
-    setEnv(TEXT("WRAPPER_BIN_DIR"), szPath, ENV_SOURCE_WRAPPER);
+    setEnv(TEXT("WRAPPER_BIN_DIR"), szPath, ENV_SOURCE_APPLICATION);
 
     if (wrapperSetWorkingDir(szPath, TRUE)) {
         free(szPath);
@@ -1692,29 +1803,17 @@ int main(int argc, char **argv) {
     size_t req;
     TCHAR **argv;
     
-    /* Create UNICODE versions of the argv array for internal use. */
-    argv = malloc(argc * sizeof *argv );
-    if(!argv) {
-        _tprintf(TEXT("Out of Memory in Main\n"));
-        appExit(1, 0, NULL);
+#ifdef FREEBSD
+    /* In the case of FreeBSD, we need to dynamically load and initialize the iconv library to work with all versions of FreeBSD. */
+    if (loadIconvLibrary()) {
+        /* Already reported. */
+        /* Don't call appExit here as we are not far enough along. */
         return 1;
     }
-    for (i = 0; i < argc; i++) {
-        req = mbstowcs(NULL, cargv[i], 0);
-        argv[i] = malloc((int)(req + 1) * sizeof(TCHAR));
-        if (!argv[i]) {
-            _tprintf(TEXT("Out of Memory in Main\n"));
-            while(--i > 0) {
-                free(argv[i]);
-            }
-            free(argv);
-            appExit(1, 0, argv);
-            return 1;
-        }
-        mbstowcs(argv[i], cargv[i], (req + 1) * sizeof(TCHAR));
-    }
-
-#endif
+#endif  
+  
+    /* we should set the locale before trying to convert cargv to argv,
+     * because there might be accentued letter in cargv */
     retLocale = _tsetlocale(LC_ALL, TEXT(""));
     if (retLocale) {
 #if defined(UNICODE)
@@ -1730,6 +1829,38 @@ int main(int argc, char **argv) {
 #endif
         */
     }
+    
+    /* Create UNICODE versions of the argv array for internal use. */
+    argv = malloc(argc * sizeof(TCHAR *));
+    if (!argv) {
+        _tprintf(TEXT("Out of Memory in Main\n"));
+        appExit(1, 0, NULL);
+        return 1;
+    }
+    for (i = 0; i < argc; i++) {
+        req = mbstowcs(NULL, cargv[i], MBSTOWCS_QUERY_LENGTH);
+        if (req == (size_t)-1) {
+            _tprintf(TEXT("Encoding problem with arguments in Main\n"));
+            free(argv);
+            appExit(1, 0, NULL);
+            return 1;
+        }
+        argv[i] = malloc(sizeof(TCHAR) * (req + 1));
+        if (!argv[i]) {
+            _tprintf(TEXT("Out of Memory in Main\n"));
+            while (--i > 0) {
+                free(argv[i]);
+            }
+            free(argv);
+            appExit(1, 0, NULL);
+            return 1;
+        }
+        mbstowcs(argv[i], cargv[i], req + 1);
+        argv[i][req] = TEXT('\0'); /* Avoid bufferflows caused by badly encoded characters. */
+    }
+
+#endif
+
     if (wrapperInitialize()) {
         appExit(1, argc, argv);
         return 1; /* For compiler. */
