@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2013 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -42,9 +42,12 @@ barf
 #include <tlhelp32.h>
 #include <winnt.h>
 #include <Sddl.h>
-#include "wrapper_i18n.h"
+#include "loggerjni.h"
 #include "wrapperjni.h"
 
+/* The largest possible command line length on Windows. */
+#define MAX_COMMAND_LINE_LEN 32766
+
 /* MS Visual Studio 8 went and deprecated the POXIX names for functions.
  *  Fixing them all would be a big headache for UNIX versions. */
 #pragma warning(disable : 4996)
@@ -545,9 +548,8 @@ jobject createWrapperUserForProcess(JNIEnv *env, DWORD processId, jboolean group
     return wrapperUser;
 }
 
+HMODULE kernel32Mod;
 void loadDLLProcs() {
-    HMODULE kernel32Mod;
-
     if ((kernel32Mod = GetModuleHandle(TEXT("KERNEL32.DLL"))) == NULL) {
         _tprintf(TEXT("WrapperJNI Error: Unable to load KERNEL32.DLL: %s\n"), getLastErrorText());
         flushall();
@@ -638,7 +640,10 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeInit(JNIEnv *env, jclass jC
             flushall();
         }
     }
-    initCommon(env, jClassWrapperManager);
+    if (initCommon(env, jClassWrapperManager)) {
+        /* Failed.  An exception will have been thrown. */
+        return;
+    }
 
     osVer.dwOSVersionInfoSize = sizeof(osVer);
     if (GetVersionEx(&osVer)) {
